/*
 * Copyright 2011-2011 Christos KK Loverdos
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.ckkloverdos

package object maybe {
  @inline
  final def ignoreExceptions(f: => Any): Unit = {
    try f
    catch { case _ => } finally {}
  }
  
  def effect[A](f: => A)(_catch: => Unit)(_finally: => Unit): Maybe[A] = {
    try {
      f match {
        case null => NoVal
        case a    => Just(a)
      }
    } catch {
      case e: Throwable =>
        ignoreExceptions (_catch)
        Failed(e, "effect{}")
    } finally {
      ignoreExceptions(_finally)
    }
  }

  @inline
  final def safe[A](f: => A): Maybe[A] = Maybe(f)
}
